@@ -1,5 +1,6 @@
 #pragma once
 
+#include "array.h"
 #include "helpers.h"
 #include "ldt_base.h"
 #include <algorithm> // std::sort, std::stable_sort
@@ -8,41 +9,30 @@
 #include <string>
 #include <vector>
 
-// #pragma region Exceptions
-
-/// @brief inversion of a singular Matrix
-static const char exp_mat_sin[] = "mat_sin";
-
-// #pragma endregion
-
 namespace ldt {
 
-/// @brief A range with an start and end indices
-class LDT_EXPORT IndexRange {
+/// @brief An iterator class for rows (matrix is column major)
+template <class Tw = Tv> class LDT_EXPORT MatIterator {
+
+private:
+  /// @brief pointer to the current element in row
+  Tw *ptr;
+
+  /// @brief stride between elements in the row
+  Ti stride;
 
 public:
-  /// @brief Starting index of the range
-  Ti StartIndex = 0;
+  MatIterator<Tw>(Tw *p, Ti s);
 
-  /// @brief Ending index of the range
-  Ti EndIndex = 0;
+  MatIterator<Tw> &operator++();
 
-  /// @brief Initializes a new instance of the class
-  IndexRange(){};
+  MatIterator<Tw> operator++(Ti);
 
-  /// @brief Initializes a new instance of the class
-  /// @param start Starting index of the range
-  /// @param end Ending index of the range
-  IndexRange(Ti start, Ti end);
+  bool operator==(const MatIterator<Tw> &rhs) const;
 
-  /// @brief Determines whether this range is invalid (start > end, start < 0,
-  /// end < 0)
-  /// @return true if the range is not valid, false otherwise
-  bool IsNotValid() const;
+  bool operator!=(const MatIterator<Tw> &rhs) const;
 
-  /// @brief Number of elements in this range. It is 1 if start==end
-  /// @return Number of elements in this range
-  Ti Count() const;
+  Tw &operator*();
 };
 
 /// @brief A matrix with an array
@@ -106,6 +96,45 @@ public:
 
   // #pragma endregion
 
+  // #pragma region Iterators
+
+  /// @brief Returns an iterator to the beginning of the matrix.
+  /// @return The iterator.
+  Tw *begin();
+
+  /// @brief Returns an iterator to the end of the matrix.
+  /// @return The iterator
+  Tw *end();
+
+  /// @brief Returns an iterator to the beginning of a specific column of the
+  /// matrix.
+  /// @param col The column
+  /// @return The iterator
+  /// @example for (auto it = m.ColBegin(j); it != m.ColBegin(j); ++it) {}
+  /// Use nested loops for iterating over multiple columns
+  Tw *ColBegin(Ti col);
+
+  /// @brief Returns an iterator to the end of a specific column of the matrix.
+  /// @param col The column
+  /// @return The iterator
+  Tw *ColEnd(Ti col);
+
+  /// @brief Returns an iterator to the beginning of a specific row of the
+  /// matrix.
+  /// @param row The row
+  /// @return The iterator
+  /// @example for (auto it = m.RowBegin(j); it != m.RowEnd(j); ++it) {}
+  /// Use nested loops for iterating over multiple columns.
+  MatIterator<Tw> RowBegin(Ti row);
+
+  /// @brief Returns an iterator to the end of a specific row of the
+  /// matrix.
+  /// @param row The row
+  /// @return The iterator
+  MatIterator<Tw> RowEnd(Ti row);
+
+  // #pragma endregion
+
   // #pragma region Data
 
   /// @brief changes the number of rows and columns of the Matrix. Old and new
@@ -299,6 +328,18 @@ public:
   /// @return range of data in the row
   IndexRange GetRangeRow(bool &hasMissing, Ti i = 0) const;
 
+  /// @brief Interpolates missing data in a specific column
+  /// @param count On exit it is the number of interpolated points
+  /// @param j Index of the column
+  /// @return Range of data in that column
+  IndexRange InterpolateColumn(Ti &count, Ti j = 0);
+
+  /// @brief Interpolates missing data in a specific row
+  /// @param count On exit it is the number of interpolated points
+  /// @param i Index of the row
+  /// @return Range of data in that row
+  IndexRange InterpolateRow(Ti &count, Ti i = 0);
+
   /// @brief Gets the range of data in the columns
   /// @param anyColumnHasMissing (updated on exit) true if missing observation
   /// is found in any column
@@ -1772,6 +1813,9 @@ public:
   bool All(Tw value) const;
 };
 
+extern template class ldt::MatIterator<Ti>;
+extern template class ldt::MatIterator<Tv>;
+
 extern template class ldt::Matrix<Ti>;
 extern template class ldt::Matrix<Tv>;
 
