@@ -276,11 +276,11 @@ public:
   std::vector<EstimationKeep *> All;
 
   /// @brief If requested in the options, it is the CDFs at the given points
-  std::vector<RunningWeightedMean> Cdfs;
+  std::vector<RunningMoments<1, true, true, Tv>> Cdfs;
 
   /// @brief If requested in the options, it is the first 4 moments of the
   /// Mixture4 distribution
-  RunningWeighted4 Mixture4;
+  RunningMoments<4, true, true, Tv> Mixture4;
 
   /// @brief If requested in the options, it is an array of size 2 [min,max],
   /// indicating the extreme bounds
@@ -288,7 +288,7 @@ public:
 
   /// @brief If requested in the options, it has the information about the
   /// inclusion weights of dependent and exogenous variables
-  std::vector<RunningWeightedMean> InclusionsInfo;
+  std::vector<RunningMoments<1, true, false, Tv>> InclusionsInfo;
 
   /// @brief Pointer to the given \ref SearchItems in the constructor
   const SearchItems *pItems = nullptr;
@@ -572,7 +572,7 @@ public:
   /// @param result A place to save the result
   void CombineInclusionWeights(Ti index1, Ti index2, Ti index3,
                                const std::vector<SearcherSummary *> &summaries,
-                               RunningWeightedMean &result);
+                               RunningMoments<1, true, false, Tv> &result);
 
   /// @brief Combines CDF at a specific point for a specific item
   /// @param index1 Metric index of the item
@@ -583,7 +583,7 @@ public:
   /// @param result A place to save the result
   void CombineCdfAt(Ti index1, Ti index2, Ti index3, Ti cdfIndex,
                     const std::vector<SearcherSummary *> &summaries,
-                    RunningWeightedMean &result);
+                    RunningMoments<1, true, true, Tv> &result);
 
   /// @brief Combines extreme bounds for a specific item
   /// @param index1 Metric index of the item
@@ -603,7 +603,7 @@ public:
   /// @param result A place to save the result
   void CombineMixture(Ti index1, Ti index2, Ti index3,
                       const std::vector<SearcherSummary *> &summaries,
-                      RunningWeighted4 &result);
+                      RunningMoments<4, true, true, Tv> &result);
 };
 
 } // namespace ldt
