@@ -9,11 +9,11 @@
 using namespace ldt;
 
 SurSimulation::SurSimulation(Ti N, Ti m, Ti k, Tv trainRatio, Ti trainFixSize,
-                             const std::vector<ScoringType> &measuresOut,
+                             const std::vector<ScoringType> &metricsOut,
                              bool isRestricted, Ti maxSigSearchIter,
                              PcaAnalysisOptions *pcaOptionsY,
                              PcaAnalysisOptions *pcaOptionsX) {
-  pMeasuresOut = &measuresOut;
+  pMetricsOut = &metricsOut;
 
   mTrainPerc = trainRatio;
   mTrainFixSize = trainFixSize;
@@ -28,7 +28,7 @@ SurSimulation::SurSimulation(Ti N, Ti m, Ti k, Tv trainRatio, Ti trainFixSize,
   Split_d = std::unique_ptr<Ti[]>(new Ti[Split.WorkSizeI]);
 
   auto doForcVariance = false;
-  for (auto &s : measuresOut)
+  for (auto &s : metricsOut)
     if (Scoring::RequiresVariance(s)) {
       doForcVariance = true;
       break;
@@ -37,9 +37,12 @@ SurSimulation::SurSimulation(Ti N, Ti m, Ti k, Tv trainRatio, Ti trainFixSize,
   Model = SurExtended(
       N0, m, k, isRestricted, false, // NANs must be removed before
       false, N1, maxSigSearchIter, doForcVariance, pcaOptionsY, pcaOptionsX);
+
   WorkSize = Split.StorageSize + Model.StorageSize + Model.WorkSize;
+  // I have generated Split.WorkSizeI here
   WorkSize += N1 * m; // for errors matrix
-  StorageSize = m * (Ti)measuresOut.size();
+
+  StorageSize = m * (Ti)metricsOut.size();
 }
 
 void SurSimulation::AddError(std::string state) {
@@ -56,39 +59,39 @@ void SurSimulation::AddError(std::string state) {
 
 Tv sumScores(const ScoringType &e, const Ti &length, const Tv *actuals,
              const Tv *errors, const Tv *means, const Tv *variances) {
-  Tv avg = 0;
+  Tv sum = 0;
   switch (e) {
   case ldt::ScoringType::kDirection:
     throw std::logic_error("not implemented (direction)");
   case ldt::ScoringType::kSign:
     for (Ti i = 0; i < length; i++)
-      avg += actuals[i] * means[i] > 0 ? 1 : 0;
+      sum += actuals[i] * means[i] > 0 ? 1 : 0;
     break;
   case ldt::ScoringType::kMae:
     for (Ti i = 0; i < length; i++)
-      avg += std::abs(errors[i]);
+      sum += std::abs(errors[i]);
     break;
-  case ldt::ScoringType::kScaledMae:
+  case ldt::ScoringType::kMape:
     for (Ti i = 0; i < length; i++)
-      avg += std::abs(errors[i] / actuals[i]);
+      sum += std::abs(errors[i] / actuals[i]);
     break;
   case ldt::ScoringType::kRmse:
     for (Ti i = 0; i < length; i++)
-      avg += std::pow(errors[i], 2.0);
+      sum += std::pow(errors[i], 2.0);
     break;
-  case ldt::ScoringType::kScaledRmse:
+  case ldt::ScoringType::kRmspe:
     for (Ti i = 0; i < length; i++)
-      avg += std::pow(errors[i] / actuals[i], 2.0);
+      sum += std::pow(errors[i] / actuals[i], 2.0);
     break;
   case ldt::ScoringType::kCrps:
     for (Ti i = 0; i < length; i++)
-      avg += Scoring::GetScoreCrpsNormal(errors[i], 0, std::sqrt(variances[i]));
+      sum += Scoring::GetScoreCrpsNormal(errors[i], 0, std::sqrt(variances[i]));
     break;
   default:
     throw std::logic_error("not implemented (averaging scores)");
   }
 
-  return avg;
+  return sum;
 }
 
 void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
@@ -104,7 +107,8 @@ void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
   Ti k = data.ColsCount - m;
   // Ti km = k * m;
 
-  Results = Matrix<Tv>(storage, (Ti)pMeasuresOut->size(), m);
+  Results = Matrix<Tv>(storage, (Ti)pMetricsOut->size(), m);
+  Results.SetValue(0.0);
 
   std::mt19937 eng;
   if (seed == 0) {
@@ -112,18 +116,21 @@ void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
     eng = std::mt19937(rdev());
   } else
     eng = std::mt19937(seed);
+
   Ti p = 0;
   Split.Calculate(data, &work[p], mTrainPerc, mTrainFixSize);
   p += Split.StorageSize;
   auto model_storage = &work[p];
   p += Model.StorageSize;
+  auto model_work = &work[p];
+  p += Model.WorkSize;
+  auto error_work = &work[p];
+  p += Split.Sample1.RowsCount * m;
 
   auto newY = Matrix<Tv>(Split.Sample1.Data, Split.Sample1.RowsCount, m);
   auto newX = Matrix<Tv>(&Split.Sample1.Data[m * Split.Sample1.RowsCount],
                          Split.Sample1.RowsCount, k);
-  auto errors = Matrix<Tv>(&work[p], Split.Sample1.RowsCount, m);
-
-  auto other_work = &work[p];
+  auto errors = Matrix<Tv>(error_work, Split.Sample1.RowsCount, m);
 
   Ti i, j, s, invalidCounts = 0;
   ValidCounts = 0;
@@ -136,19 +143,13 @@ void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
     Split.Shuffle(data, Split_d.get(), eng);
 
     try {
-      Model.Calculate(Split.Sample0, m, model_storage, other_work, R,
+      Model.Calculate(Split.Sample0, m, model_storage, model_work, R,
                       sigSearchMaxProb, &newX, nullptr);
       if (Model.Model.condition_number > maxCondNum)
         throw std::logic_error("Model check failed: Maximum CN");
     } catch (std::exception &ex) {
       AddError(ex.what());
       continue;
-    } catch (std::string &ex) {
-      AddError(ex.c_str());
-      continue;
-    } catch (const char *ex) {
-      AddError(ex);
-      continue;
     } catch (...) {
       AddError("unknown error!");
       continue;
@@ -171,20 +172,17 @@ void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
       throw std::logic_error("Model check failed: Minimum Valid Simulations");
 
     i = -1;
-    for (auto &e : *pMeasuresOut) {
+    for (auto &e : *pMetricsOut) {
       i++;
       for (j = 0; j < m; j++) {
         if (cancel)
           return;
         s = j * errors.RowsCount;
-        Results.Set0(i, j,
-                     sumScores(e, errors.RowsCount, &newY.Data[s],
-                               &errors.Data[s],
-                               &Model.Projections.Means.Data[s],
-                               &Model.Projections.Variances
-                                    .Data[s])); // TODO: individual averages
-                                                // might reduce rounding errors
-                                                // (just use for RMSE)
+        Results.Set_Plus0(i, j,
+                          sumScores(e, errors.RowsCount, &newY.Data[s],
+                                    &errors.Data[s],
+                                    &Model.Projections.Means.Data[s],
+                                    &Model.Projections.Variances.Data[s]));
       }
     }
   }
@@ -196,4 +194,22 @@ void SurSimulation::Calculate(Matrix<Tv> &data, Ti m, Tv *storage, Tv *work,
     throw std::logic_error("Model check failed: Minimum Valid Simulations");
 
   Results.Divide_in((Tv)ValidCounts);
+
+  // sqrt for RMSE or RMSPE
+  i = -1;
+  for (auto &e : *pMetricsOut) {
+    i++;
+
+    if (e == ScoringType::kRmse || e == ScoringType::kRmspe)
+      for (j = 0; j < m; j++) {
+        s = j * errors.RowsCount;
+        Results.Set0(i, j, std::sqrt(Results.Get0(i, j)));
+      }
+
+    // convert to percentage
+    if (e == ScoringType::kMape || e == ScoringType::kRmspe)
+      for (j = 0; j < m; j++) {
+        Results.Set0(i, j, Results.Get0(i, j) * 100);
+      }
+  }
 }
