@@ -289,8 +289,13 @@ public:
 
   Ti StorageSize = 0;
 
+  /// @brief Determines where forecasts start in \ref Forecast
   Ti StartIndex = 0;
 
+  /// @brief Determines the number of observations and the first observation in
+  /// forecast. Use it e.g. for setting the frequency.
+  Ti StartDiff = 0;
+
   Ti mHorizon = 0;
   Ti mDoVariance = 0;
   bool mCoefUncertainty = false;
@@ -363,6 +368,7 @@ public:
   VarmaRestrictionType mRestriction;
   PcaAnalysisOptions *pPcaOptionsY = nullptr;
   PcaAnalysisOptions *pPcaOptionsX = nullptr;
+  Ti mHorizon = 0;
 
   Ti WorkSize = 0;
   Ti StorageSize = 0;
@@ -444,18 +450,16 @@ public:
 
   const std::vector<Ti> *pHorizons = nullptr;
 
-  const std::vector<ScoringType> *pMeasures = nullptr;
-
-  // bool mKeepDetails;
+  const std::vector<ScoringType> *pMetrics = nullptr;
 
   Varma Model;
   VarmaForecast Forecast;
   VarmaExtended EModel;
 
-  /// @brief at(i)->[j,h] => i-th measure, j-th variable, h-th horizon
+  /// @brief at(i)->[j,h] => i-th metric, j-th variable, h-th horizon
   std::vector<Matrix<Tv>> Results;
 
-  /// @brief Results aggregated by horizon [i,j] => i-th measure, j-th variable
+  /// @brief Results aggregated by horizon [i,j] => i-th metric, j-th variable
   /// (aggregated over horizons). Note that it might be a weighted average due
   /// to different number of evaluations over different horizon. For RMSE, it
   /// is different furthermore, because of its nonlinearity nature.
@@ -463,13 +467,17 @@ public:
 
   Ti ValidCounts = 0;
 
-  // std::vector<VarmaSimulationDetail*>* Details;
+  bool KeepDetails = false;
+
+  /// @brief The items are: sample end, metric index, horizon, target index,
+  /// last value, actual value, forecast, forecast error, std
+  std::vector<std::tuple<Ti, Ti, Ti, Ti, Tv, Tv, Tv, Tv, Tv>> Details;
 
   VarmaSimulation(){};
 
   VarmaSimulation(
       const VarmaSizes &sizes, Ti count, const std::vector<Ti> &horizons,
-      const std::vector<ScoringType> &measures,
+      const std::vector<ScoringType> &metrics,
       LimitedMemoryBfgsbOptions *optimOptions = nullptr,
       bool isExtended = false,
       VarmaRestrictionType restriction = VarmaRestrictionType::kMaFinal,
@@ -517,10 +525,9 @@ class LDT_EXPORT VarmaSearcher : public Searcher {
 
 public:
   VarmaSearcher(SearchOptions &searchOptions, const SearchItems &searchItems,
-                const SearchMeasureOptions &measures,
+                const SearchMetricOptions &metrics,
                 const SearchModelChecks &checks, Ti sizeG,
-                const std::vector<std::vector<Ti>> &groupIndexMap,
-                const std::vector<Ti> &groupSizes, Ti fixFirstG,
+                const std::vector<std::vector<Ti>> &groupIndexMap, Ti fixFirstG,
                 DatasetTs<true> &source, const VarmaSizes sizes,
                 const std::vector<Ti> &exoIndexes, Matrix<Tv> *forLowerBounds,
                 Matrix<Tv> *forUpperBounds,
@@ -537,15 +544,13 @@ public:
 
   std::vector<Searcher *> Searchers;
 
-  std::vector<Ti> GroupSizes;
-
   Matrix<Tv> ForecastLowers;
   Matrix<Tv> ForecastUppers;
 
   VarmaModelset(){};
 
   VarmaModelset(SearchOptions &searchOptions, SearchItems &searchItems,
-                SearchMeasureOptions &measures, SearchModelChecks &checks,
+                SearchMetricOptions &metrics, SearchModelChecks &checks,
                 const std::vector<Ti> &sizes,
                 std::vector<std::vector<Ti>> &groupIndexMap,
                 DatasetTs<true> &source, std::vector<Ti> varmaMaxParameters6,
